@@ -5,11 +5,11 @@
 # (see LICENSE.txt for details)
 """Unit Testing widget."""
 
-from __future__ import with_statement
-
 # Standard library imports
+import ast
 import copy
 import os.path as osp
+import subprocess
 import sys
 
 # Third party imports
@@ -19,22 +19,16 @@ from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.config.base import get_conf_path, get_translation
 from spyder.utils import icon_manager as ima
 from spyder.plugins.variableexplorer.widgets.texteditor import TextEditor
-from spyder.py3compat import PY3
 
 # Local imports
 from spyder_unittest.backend.frameworkregistry import FrameworkRegistry
 from spyder_unittest.backend.noserunner import NoseRunner
+from spyder_unittest.backend.pytestrunner import PyTestRunner
 from spyder_unittest.backend.runnerbase import Category, TestResult
 from spyder_unittest.backend.unittestrunner import UnittestRunner
 from spyder_unittest.widgets.configdialog import Config, ask_for_config
 from spyder_unittest.widgets.datatree import TestDataModel, TestDataView
 
-# This import uses Python 3 syntax, so importing it under Python 2 throws
-# a SyntaxError which means that the plugin's check_compatibility method
-# will never run.
-if PY3:
-    from spyder_unittest.backend.pytestrunner import PyTestRunner
-
 # This is needed for testing this module as a stand alone script
 try:
     _ = get_translation('spyder_unittest')
@@ -42,11 +36,8 @@ except KeyError:
     import gettext
     _ = gettext.gettext
 
-# Supported testing framework
-if PY3:
-    FRAMEWORKS = {NoseRunner, PyTestRunner, UnittestRunner}
-else:
-    FRAMEWORKS = {NoseRunner, UnittestRunner}
+# Supported testing frameworks
+FRAMEWORKS = {NoseRunner, PyTestRunner, UnittestRunner}
 
 
 class UnitTestWidgetActions:
@@ -78,6 +69,10 @@ class UnitTestWidget(PluginMainWidget):
         Configuration for running tests, or `None` if not set.
     default_wdir : str
         Default choice of working directory.
+    dependencies : dict or None
+        Cached dependencies, as returned by `self.get_versions()`.
+    environment_for_dependencies : str or None
+        Python interpreter for which `self.dependencies` is valid.
     framework_registry : FrameworkRegistry
         Registry of supported testing frameworks.
     pre_test_hook : function returning bool or None
@@ -98,6 +93,7 @@ class UnitTestWidget(PluginMainWidget):
         Arguments are file name and line number (zero-based).
     """
 
+    CONF_SECTION = 'unittest'
     VERSION = '0.0.1'
 
     sig_finished = Signal()
@@ -109,12 +105,14 @@ class UnitTestWidget(PluginMainWidget):
         super().__init__(name, plugin, parent)
 
         self.config = None
-        self.pythonpath = None
         self.default_wdir = None
+        self.dependencies = None
+        self.environment_for_dependencies = None
+        self.output = None
         self.pre_test_hook = None
+        self.pythonpath = None
         self.testrunner = None
 
-        self.output = None
         self.testdataview = TestDataView(self)
         self.testdatamodel = TestDataModel(self)
         self.testdataview.setModel(self.testdatamodel)
@@ -255,16 +253,54 @@ class UnitTestWidget(PluginMainWidget):
             te.show()
             te.exec_()
 
+    def get_versions(self, use_cached):
+        """
+        Return versions of frameworks and their plugins.
+
+        If `use_cached` is `True` and `self.environment_for_dependencies`
+        equals the Python interpreter set by the user in the Preferences,
+        then return the cached information in `self.dependencies`.
+
+        Otherwise, run the `print_versions.py` script in the target
+        environment to retrieve the dependencyy information. Store that
+        information in `self.dependencies` and return it.
+
+        Parameters
+        ----------
+        use_cached : bool
+            Whether to use the cached information, if possible.
+
+        Returns
+        -------
+        dict
+            Dependency information as returned by `print_versions.py`
+        """
+        executable = self.get_conf('executable', section='main_interpreter')
+        if use_cached and self.environment_for_dependencies == executable:
+            return self.dependencies
+
+        script = osp.join(osp.dirname(__file__), osp.pardir, 'backend',
+                          'workers', 'print_versions.py')
+        process = subprocess.run([executable, script],
+                                 capture_output=True, text=True)
+        self.dependencies = ast.literal_eval(process.stdout)
+        self.environment_for_dependencies = executable
+        return self.dependencies
+
     def show_versions(self):
         """Show versions of frameworks and their plugins"""
+        all_info = self.get_versions(use_cached=False)
         versions = [_('Versions of frameworks and their installed plugins:')]
-        for name, runner in sorted(self.framework_registry.frameworks.items()):
-            version = (runner.get_versions(self) if runner.is_installed()
-                       else None)
-            versions.append('\n'.join(version) if version else
-                            '{}: {}'.format(name, _('not available')))
+        for name, info in all_info.items():
+            if not info['available']:
+                versions.append('{}: {}'.format(name, _('not available')))
+            else:
+                version = f'{name} {info["version"]}'
+                plugins = [f'   {name} {version}'
+                           for name, version in info['plugins'].items()]
+                versions.append('\n'.join([version] + plugins))
         QMessageBox.information(self, _('Dependencies'),
-                                _('\n\n'.join(versions)))
+                                '\n\n'.join(versions))
 
     def configure(self):
         """Configure tests."""
@@ -273,7 +309,8 @@ class UnitTestWidget(PluginMainWidget):
         else:
             oldconfig = Config(wdir=self.default_wdir)
         frameworks = self.framework_registry.frameworks
-        config = ask_for_config(frameworks, oldconfig, parent=self)
+        versions = self.get_versions(use_cached=True)
+        config = ask_for_config(frameworks, oldconfig, versions, parent=self)
         if config:
             self.config = config
 
@@ -341,8 +378,13 @@ class UnitTestWidget(PluginMainWidget):
         self.testrunner.sig_testresult.connect(self.tests_yield_result)
         self.testrunner.sig_stop.connect(self.tests_stopped)
 
+        cov_path = self.get_conf('current_project_path', default='None',
+                                 section='project_explorer')
+        # config returns 'None' as a string rather than None
+        cov_path = config.wdir if cov_path == 'None' else cov_path
+        executable = self.get_conf('executable', section='main_interpreter')
         try:
-            self.testrunner.start(config, pythonpath)
+            self.testrunner.start(config, cov_path, executable, pythonpath)
         except RuntimeError:
             QMessageBox.critical(self,
                                  _("Error"), _("Process failed to start"))
@@ -380,7 +422,7 @@ class UnitTestWidget(PluginMainWidget):
             button.clicked.connect(
                 lambda checked: self.maybe_configure_and_start())
 
-    def process_finished(self, testresults, output):
+    def process_finished(self, testresults, output, normal_exit):
         """
         Called when unit test process finished.
 
@@ -388,18 +430,22 @@ class UnitTestWidget(PluginMainWidget):
 
         Parameters
         ----------
-        testresults : list of TestResult or None
-            `None` indicates all test results have already been transmitted.
+        testresults : list of TestResult
+            Test results reported when the test process finished.
         output : str
+            Output from the test process.
+        normal_exit : bool
+            Whether test process exited normally.
         """
         self.output = output
         self.set_running_state(False)
         self.testrunner = None
         self.show_log_action.setEnabled(bool(output))
-        if testresults is not None:
-            self.testdatamodel.testresults = testresults
+        self.testdatamodel.add_testresults(testresults)
         self.replace_pending_with_not_run()
         self.sig_finished.emit()
+        if not normal_exit:
+            self.set_status_label(_('Test process exited abnormally'))
 
     def replace_pending_with_not_run(self):
         """Change status of pending tests to 'not run''."""
